package lib.ui.reactive;

import util.Pos;
import util.Rect;
import util.Color;
import util.Vec.Vec2;
import kernel.ui.WindowContext;

using tink.CoreApi;

class ReactiveUI {
	private final context:WindowContext;
	private final children:Array<UIElement>;
	private var elementMap: Map<Rect,UIElement> = new Map();

	public function new(context:WindowContext, children:Array<UIElement>) {
		this.context = context;
		this.children = children;

		for (child in children) {
			child.changed.handle(_ -> {
				context.reset();
				render();
			});
		}

		setupEvents();
	}

	private function setupEvents() {
		context.onClick.handle(params ->{
			for (k => v in elementMap){
				if (k.isInside(params.pos)){
					if (v.eventListner.onClick != null){
						v.eventListner.onClick.invoke(params);
					}
				}
			}
		});

		// context.onKey.handle(params ->{
		// 	for (k => v in elementMap){
		// 		if (k.isInside(params.pos)){
		// 			if (v.eventListner.onKey != null){
		// 				v.eventListner.onKey.invoke(params);
		// 			}
		// 		}
		// 	}
		// });

		// context.onKeyUp.handle(params ->{
		// 	for (k => v in elementMap){
		// 		if (k.isInside(params.pos)){
		// 			if (v.eventListner.onKeyUp != null){
		// 				v.eventListner.onKeyUp.invoke(params);
		// 			}
		// 		}
		// 	}
		// });

		// context.onMouseDrag.handle(params ->{
		// 	for (k => v in elementMap){
		// 		if (k.isInside(params.pos)){
		// 			if (v.eventListner.onMouseDrag != null){
		// 				v.eventListner.onMouseDrag.invoke(params);
		// 			}
		// 		}
		// 	}
		// });

		context.onMouseScroll.handle(params ->{
			for (k => v in elementMap){
				if (k.isInside(params.pos)){
					if (v.eventListner.onMouseScroll != null){
						v.eventListner.onMouseScroll.invoke(params);
					}
				}
			}
		});

		context.onMouseUp.handle(params ->{
			for (k => v in elementMap){
				if (k.isInside(params.pos)){
					if (v.eventListner.onMouseUp != null){
						v.eventListner.onMouseUp.invoke(params);
					}
				}
			}
		});

		// context.onPaste.handle(params ->{
		// 	for (k => v in elementMap){
		// 		if (k.isInside(params.pos)){
		// 			if (v.eventListner.onPaste != null){
		// 				v.eventListner.onPaste.invoke(params);
		// 			}
		// 		}
		// 	}
		// });

	}

	public function render() {
		var size = context.getSize();
		var result = renderChildren(children, size);
		this.elementMap = result.map;

		writeToContext(result.canvas);
	}

	private function writeToContext(screen:Canvas) {
		var currentBg:Color = Black;
		var currentFg:Color = White;

		var currentLine = 0;

		context.setBackgroundColor(currentBg);
		context.setTextColor(currentFg);
		context.setCursorPos(0, 0);

		for (key => pixel in screen) {
			if (key.y != currentLine) {
				currentLine = key.y;
				context.setCursorPos(key.x, key.y);
			}

			if (pixel == null) {
				context.write(' ');
			} else {
				if (pixel.bg != currentBg) {
					context.setBackgroundColor(pixel.bg);
					currentBg = pixel.bg;
				}

				if (pixel.textColor != currentFg) {
					context.setTextColor(pixel.textColor);
					currentFg = pixel.textColor;
				}

				context.write(pixel.char);
			}
		}
	}

	public static function renderChildren(children:Array<UIElement>, bounds:Vec2<Int>):{canvas: Canvas, map: Map<Rect,UIElement>} {
		var canvas:Canvas = new Canvas();
		var elementMap: Map<Rect,UIElement> = new Map();

		var writePoint:Pos = {x: 0, y: 0};

		for (child in children) {
			if (bounds.y - writePoint.y <= 0) {
				// No more space to render children
				break;
			}

			var childRender = child.render({
				x: bounds.x,
				y: bounds.y - writePoint.y
			},writePoint);

			canvas.combine(childRender, writePoint);
			elementMap.set(new Rect(writePoint,{x: childRender.maxWidth() + writePoint.x, y: writePoint.y + (childRender.height() - 1)}),child);

			writePoint = {x: 0, y: writePoint.y + childRender.height()};
		}

		return {canvas: canvas,map: elementMap};
	}
}
